# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution - module extension
#    Copyright (C) 2010- O4SB (<http://openforsmallbusiness.co.nz>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
'''Implements partner specific product codes and names'''

from osv import osv, fields

class product_template(osv.osv):
    '''Over ride product product to add Partner Info, note column name
    is a bit weird because product.product inherits product.template and
    if we just called partner_ids it would break'''
    _inherit = 'product.template'
    
    _columns = {
        'partner_tmpl_ids': fields.one2many('product.partnerinfo', 'product_tmpl_id',
                                       'Partner Information'),
        }
    
product_template()

class product_product(osv.osv):
    '''Over ride product product to add Partner Info'''
    _inherit = 'product.product'            
    
    _columns = {
        'partner_ids': fields.one2many('product.partnerinfo', 'product_id',
                                       'Partner Information'),               
        }
    
    def name_get(self, cr, user, ids, context=None):
        '''Returns name when used in fields, will look up against
        product_partnerinfo for a match before returning defaults
        '''
        if context is None:
            context = {}
        if not len(ids):
            return []
        def _name_get(prod_dict):
            name = prod_dict.get('name','')
            code = prod_dict.get('default_code', False)
            if code:
                name = '[%s] %s' % (code, name)
            if prod_dict.get('variants'):
                name = name + ' - %s' % (prod_dict['variants'],)
            return (prod_dict['id'], name)

        partner_id = context.get('partner_id', False)
        partinfo_obj = self.pool.get('product.partnerinfo')
        
        if partner_id:
            partner = self.pool.get('res.partner').browse(cr, user, partner_id)
            while partner.parent_id:
                partner = partner.parent_id
        else:
            partner = False

        result = []
        for product in self.browse(cr, user, ids, context=context):
            partinfo_id = partinfo_obj.search(cr, user, [('name', '=', partner_id), '|', 
                                              ('product_tmpl_id', '=', product.product_tmpl_id.id),
                                              ('product_id', '=', product.id)])
            if partinfo_id:
                partinfo = partinfo_obj.browse(cr, user, partinfo_id[0])
                mydict = {
                          'id': product.id,
                          'name': partinfo.product_name or product.name,
                          'default_code': partinfo.product_code or product.default_code,
                          }
                result.append(_name_get(mydict))
            else:
                mydict = {
                          'id': product.id,
                          'name': product.name,
                          'default_code': product.default_code,
                          }
                result.append(_name_get(mydict))
        return result
    
    def _get_partner_code_name(self, cr, uid, ids, product, partner_id, context=None):
        '''total override of product/product.py function for product_product class.
        Params unchanged however function now access new partnerinfo object and
        ignores supplierinfo.  Return changed to include variants on partner specific names'''
        partinfo_obj = self.pool.get('product.partnerinfo')
        partner = self.pool.get('res.partner').browse(cr, uid, partner_id)
        
        while partner.parent_id:
            partner = partner.parent_id
            
        partinfo_id = partinfo_obj.search(
                            cr, uid, [('name', '=', partner.id),
                                      '|', ('product_id', '=', product.id),
                                      ('product_tmpl_id', '=', product.product_tmpl_id.id)])
        if partinfo_id:
            partinfo = partinfo_obj.browse(cr, uid, partinfo_id[0])
            return {'code': partinfo.product_code or product.default_code, 
                    'name': partinfo.product_name or product.name, 
                    'variants': product.variants}
        return {'code': product.default_code, 'name': product.name, 'variants': product.variants}

product_product()    
    
class product_partnerinfo(osv.osv):
    '''Product Partner Info'''
    _name = 'product.partnerinfo'
    _desc = __doc__
    
    _columns = {
        'name' : fields.many2one('res.partner', 'Partner', required=True, 
                                 domain=[('parent_id', '=', False)], ondelete='cascade'),
        'product_name': fields.char('Partner Product Name', size=128, 
                                    help="This partner's product name will be used on "
                                         "documents. Keep empty to use the internal one."),
        'product_code': fields.char('Supplier Product Code', size=64, 
                                    help="This partner's product code will be used on "
                                         "documents. Keep empty to use the internal one."),
        'product_id' : fields.many2one('product.product', 'Product', ondelete='cascade' ),
        'product_tmpl_id' : fields.many2one('product.template', 'Product', ondelete='cascade' ),
        'company_id':fields.many2one('res.company','Company'),
        }
    
    def _check_partner(self, cr, uid, ids, context=None):
        '''Make sure the partner is a parent'''
        for partnerinfo in self.browse(cr, uid, ids, context):
            if partnerinfo.name.parent_id:
                return False
        return True
    
    def _check_product(self, cr, uid, ids, context=None):
        '''Make sure it is either a template or product, not both (or neither)'''
        for partnerinfo in self.browse(cr, uid, ids, context):
            if partnerinfo.product_id and partnerinfo.product_tmpl_id:
                return False
            if (not partnerinfo.product_id) and (not partnerinfo.product_tmpl_id):
                return False
        return True
    
    def _check_partner_product(self, cr, uid, ids, context=None):
        '''Basically check there is no conflicting entry'''
        for partnerinfo in self.browse(cr, uid, ids, context):
            search_args = [('id', '!=', partnerinfo.id),
                           ('name', '=', partnerinfo.name.id)]
            if partnerinfo.product_id:
                prod = partnerinfo.product_id
                search_args.extend(['|', ('product_tmpl_id', '=', prod.product_tmpl_id.id),
                                    ('product_id', '=', prod.id)])
            elif partnerinfo.product_tmpl_id:
                prod_tmpl = partnerinfo.product_tmpl_id
                search_args.extend([('product_tmpl_id', '=', prod_tmpl.id)])
            if self.search(cr, uid, search_args, context):
                return False
        return True
    
    _constraints = [
        (_check_partner, 'Error: Partcodes must be assigned at the parent partner level.', 
         ['name']),
        (_check_product, 'Error: You may specify a product or a product template only',
         ['product_id', 'product_tmpl_id']),
        (_check_partner_product, 'Error: An entry already exists for this '
        'partner/product combination', ['name', 'product_id', 'product_tmpl_id']),
        ]

product_partnerinfo()

class product_supplierinfo(osv.osv):
    '''Inherit product.supplierinfo and add a migrated flag'''
    _inherit = 'product.supplierinfo'
    
    _columns = {
            'migrated': fields.boolean("Migrated to Partnerinfo")
        }
    
    def action_migrate_supplierinfo(self, cr, uid, ids, context=None):
        '''Function to make it easy to migrate old supplier info.  This should work fine
        in the majority of cases but if you use parent relationships extensively and are
        currently using codes at subsidiary level then you may hit constraints'''
        supinfo_obj = self.pool.get('product.supplierinfo')
        partinfo_obj = self.pool.get('product.partnerinfo')
        supinfo_ids = supinfo_obj.search(cr, uid, ['|', ('product_name', '!=', False),
                                                   ('product_code', '!=', False),
                                                   ('migrated', '=', False)])
        if supinfo_ids:
            for supinfo in supinfo_obj.browse(cr, uid, supinfo_ids):
                vals = {'name': supinfo.name.parent_id and supinfo.parent_id.id or supinfo.name.id,
                        'product_tmpl_id': supinfo.product_id.id,
                        'product_name': supinfo.product_name,
                        'product_code': supinfo.product_code,
                        'company_id': supinfo.company_id and supinfo.company_id.id or False,
                        }
                partinfo_obj.create(cr, uid, vals)
        supinfo_ids = supinfo_obj.search(cr, uid, [])
        supinfo_obj.write(cr, uid, supinfo_ids, {'migrated': True})
        return True
    
    _defaults = {
                'migrated': True 
                }

product_supplierinfo()
